@@ -22,11 +22,10 @@
 //! [std::collections::hash_map::RandomState]: https://doc.rust-lang.org/std/collections/hash_map/struct.RandomState.html
 
 use std::borrow::Borrow;
-use std::cmp::Ordering;
 use std::collections;
 use std::collections::hash_map::RandomState;
 use std::fmt::{Debug, Error, Formatter};
-use std::hash::{BuildHasher, Hash, Hasher};
+use std::hash::{BuildHasher, Hash};
 use std::iter::{FromIterator, FusedIterator, Sum};
 use std::mem;
 use std::ops::{Add, Index, IndexMut};
@@ -933,13 +932,21 @@ where
     /// ```
     #[must_use]
     pub fn union(self, other: Self) -> Self {
-        let (mut to_mutate, to_consume) = if self.len() >= other.len() {
-            (self, other)
+        let (mut to_mutate, to_consume, use_to_consume) = if self.len() >= other.len() {
+            (self, other, false)
         } else {
-            (other, self)
+            (other, self, true)
         };
         for (k, v) in to_consume {
-            to_mutate.entry(k).or_insert(v);
+            match to_mutate.entry(k) {
+                Entry::Occupied(mut e) if use_to_consume => {
+                    e.insert(v);
+                }
+                Entry::Vacant(e) => {
+                    e.insert(v);
+                }
+                _ => {}
+            }
         }
         to_mutate
     }
@@ -1411,6 +1418,7 @@ where
 
     /// Call the provided function to modify the value if the value
     /// exists.
+    #[must_use]
     pub fn and_modify<F>(mut self, f: F) -> Self
     where
         F: FnOnce(&mut V),
@@ -1608,50 +1616,6 @@ where
 {
 }
 
-impl<K, V, S> PartialOrd for HashMap<K, V, S>
-where
-    K: Hash + Eq + Clone + PartialOrd,
-    V: PartialOrd + Clone,
-    S: BuildHasher,
-{
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        if Ref::ptr_eq(&self.hasher, &other.hasher) {
-            return self.iter().partial_cmp(other.iter());
-        }
-        self.iter().partial_cmp(other.iter())
-    }
-}
-
-impl<K, V, S> Ord for HashMap<K, V, S>
-where
-    K: Hash + Eq + Ord + Clone,
-    V: Ord + Clone,
-    S: BuildHasher,
-{
-    fn cmp(&self, other: &Self) -> Ordering {
-        if Ref::ptr_eq(&self.hasher, &other.hasher) {
-            return self.iter().cmp(other.iter());
-        }
-        self.iter().cmp(other.iter())
-    }
-}
-
-impl<K, V, S> Hash for HashMap<K, V, S>
-where
-    K: Hash + Eq,
-    V: Hash,
-    S: BuildHasher,
-{
-    fn hash<H>(&self, state: &mut H)
-    where
-        H: Hasher,
-    {
-        for i in self.iter() {
-            i.hash(state);
-        }
-    }
-}
-
 impl<K, V, S> Default for HashMap<K, V, S>
 where
     S: BuildHasher + Default,
@@ -1814,6 +1778,15 @@ pub struct Iter<'a, K, V> {
     it: NodeIter<'a, (K, V)>,
 }
 
+// We impl Clone instead of deriving it, because we want Clone even if K and V aren't.
+impl<'a, K, V> Clone for Iter<'a, K, V> {
+    fn clone(&self) -> Self {
+        Iter {
+            it: self.it.clone(),
+        }
+    }
+}
+
 impl<'a, K, V> Iterator for Iter<'a, K, V> {
     type Item = (&'a K, &'a V);
 
@@ -2123,8 +2096,15 @@ mod test {
     use crate::test::LolHasher;
     use ::proptest::num::{i16, usize};
     use ::proptest::{collection, proptest};
+    use static_assertions::{assert_impl_all, assert_not_impl_any};
     use std::hash::BuildHasherDefault;
 
+    assert_impl_all!(HashMap<i32, i32>: Send, Sync);
+    assert_not_impl_any!(HashMap<i32, *const i32>: Send, Sync);
+    assert_not_impl_any!(HashMap<*const i32, i32>: Send, Sync);
+    assert_covariant!(HashMap<T, i32> in T);
+    assert_covariant!(HashMap<i32, T> in T);
+
     #[test]
     fn safe_mutation() {
         let v1: HashMap<usize, usize> = HashMap::from_iter((0..131_072).map(|i| (i, i)));
@@ -2214,15 +2194,15 @@ mod test {
     #[test]
     fn entry_api() {
         let mut map = hashmap! {"bar" => 5};
-        map.entry(&"foo").and_modify(|v| *v += 5).or_insert(1);
+        map.entry("foo").and_modify(|v| *v += 5).or_insert(1);
         assert_eq!(1, map[&"foo"]);
-        map.entry(&"foo").and_modify(|v| *v += 5).or_insert(1);
+        map.entry("foo").and_modify(|v| *v += 5).or_insert(1);
         assert_eq!(6, map[&"foo"]);
-        map.entry(&"bar").and_modify(|v| *v += 5).or_insert(1);
+        map.entry("bar").and_modify(|v| *v += 5).or_insert(1);
         assert_eq!(10, map[&"bar"]);
         assert_eq!(
             10,
-            match map.entry(&"bar") {
+            match map.entry("bar") {
                 Entry::Occupied(entry) => entry.remove(),
                 _ => panic!(),
             }
@@ -2375,5 +2355,25 @@ mod test {
             }
             assert_eq!(0, it.len());
         }
+
+        #[test]
+        fn union(ref m1 in collection::hash_map(i16::ANY, i16::ANY, 0..100),
+                 ref m2 in collection::hash_map(i16::ANY, i16::ANY, 0..100)) {
+            let map1: HashMap<i16, i16> = FromIterator::from_iter(m1.iter().map(|(k, v)| (*k, *v)));
+            let map2: HashMap<i16, i16> = FromIterator::from_iter(m2.iter().map(|(k, v)| (*k, *v)));
+            let union_map = map1.union(map2);
+
+            for k in m1.keys() {
+                assert!(union_map.contains_key(k));
+            }
+
+            for k in m2.keys() {
+                assert!(union_map.contains_key(k));
+            }
+
+            for (k, v) in union_map.iter() {
+                assert_eq!(v, m1.get(k).or_else(|| m2.get(k)).unwrap());
+            }
+        }
     }
 }
