@@ -22,11 +22,10 @@
 //! [std::collections::hash_map::RandomState]: https://doc.rust-lang.org/std/collections/hash_map/struct.RandomState.html
 
 use std::borrow::Borrow;
-use std::cmp::Ordering;
 use std::collections::hash_map::RandomState;
 use std::collections::{self, BTreeSet};
 use std::fmt::{Debug, Error, Formatter};
-use std::hash::{BuildHasher, Hash, Hasher};
+use std::hash::{BuildHasher, Hash};
 use std::iter::FusedIterator;
 use std::iter::{FromIterator, IntoIterator, Sum};
 use std::ops::{Add, Deref, Mul};
@@ -670,47 +669,6 @@ where
 {
 }
 
-impl<A, S> PartialOrd for HashSet<A, S>
-where
-    A: Hash + Eq + Clone + PartialOrd,
-    S: BuildHasher + Default,
-{
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        if Ref::ptr_eq(&self.hasher, &other.hasher) {
-            return self.iter().partial_cmp(other.iter());
-        }
-        self.iter().partial_cmp(other.iter())
-    }
-}
-
-impl<A, S> Ord for HashSet<A, S>
-where
-    A: Hash + Eq + Clone + Ord,
-    S: BuildHasher + Default,
-{
-    fn cmp(&self, other: &Self) -> Ordering {
-        if Ref::ptr_eq(&self.hasher, &other.hasher) {
-            return self.iter().cmp(other.iter());
-        }
-        self.iter().cmp(other.iter())
-    }
-}
-
-impl<A, S> Hash for HashSet<A, S>
-where
-    A: Hash + Eq,
-    S: BuildHasher + Default,
-{
-    fn hash<H>(&self, state: &mut H)
-    where
-        H: Hasher,
-    {
-        for i in self.iter() {
-            i.hash(state);
-        }
-    }
-}
-
 impl<A, S> Default for HashSet<A, S>
 where
     S: BuildHasher + Default,
@@ -843,6 +801,15 @@ pub struct Iter<'a, A> {
     it: NodeIter<'a, Value<A>>,
 }
 
+// We impl Clone instead of deriving it, because we want Clone even if K and V aren't.
+impl<'a, A> Clone for Iter<'a, A> {
+    fn clone(&self) -> Self {
+        Iter {
+            it: self.it.clone(),
+        }
+    }
+}
+
 impl<'a, A> Iterator for Iter<'a, A>
 where
     A: 'a,
@@ -950,6 +917,16 @@ where
     }
 }
 
+impl<A, S, const N: usize> From<[A; N]> for HashSet<A, S>
+where
+    A: Hash + Eq + Clone,
+    S: BuildHasher + Default,
+{
+    fn from(arr: [A; N]) -> Self {
+        IntoIterator::into_iter(arr).collect()
+    }
+}
+
 impl<'a, A, S> From<&'a [A]> for HashSet<A, S>
 where
     A: Hash + Eq + Clone,
@@ -1068,8 +1045,13 @@ mod test {
     use crate::test::LolHasher;
     use ::proptest::num::i16;
     use ::proptest::proptest;
+    use static_assertions::{assert_impl_all, assert_not_impl_any};
     use std::hash::BuildHasherDefault;
 
+    assert_impl_all!(HashSet<i32>: Send, Sync);
+    assert_not_impl_any!(HashSet<*const i32>: Send, Sync);
+    assert_covariant!(HashSet<T> in T);
+
     #[test]
     fn insert_failing() {
         let mut set: HashSet<i16, BuildHasherDefault<LolHasher>> = Default::default();
@@ -1103,7 +1085,7 @@ mod test {
         use crate::test::MetroHashBuilder;
         for i in 0..1000 {
             let mut lhs = vec![0, 1, 2];
-            lhs.sort();
+            lhs.sort_unstable();
 
             let hasher = Ref::from(MetroHashBuilder::new(i));
             let mut iset: HashSet<_, MetroHashBuilder> = HashSet::with_hasher(hasher.clone());
@@ -1112,7 +1094,7 @@ mod test {
             }
 
             let mut rhs: Vec<_> = iset.clone().into_iter().collect();
-            rhs.sort();
+            rhs.sort_unstable();
 
             if lhs != rhs {
                 println!("iteration: {}", i);
