@@ -7,17 +7,16 @@ use std::cmp::Ordering;
 use std::mem;
 use std::ops::{Bound, RangeBounds};
 
-use sized_chunks::Chunk;
-use typenum::{Add1, Unsigned};
+use imbl_sized_chunks::Chunk;
 
-use crate::config::OrdChunkSize as NodeSize;
+pub(crate) use crate::config::ORD_CHUNK_SIZE as NODE_SIZE;
 use crate::util::{Pool, PoolClone, PoolDefault, PoolRef};
 
 use self::Insert::*;
 use self::InsertAction::*;
 
-pub(crate) const NODE_SIZE: usize = NodeSize::USIZE;
 const MEDIAN: usize = (NODE_SIZE + 1) >> 1;
+const NUM_CHILDREN: usize = NODE_SIZE + 1;
 
 pub trait BTreeValue {
     type Key;
@@ -45,17 +44,15 @@ pub trait BTreeValue {
 /// The `children` array is never empty, and always has exactly one more element than `keys`. The
 /// empty tree has no keys, and a single `None` child.
 pub(crate) struct Node<A> {
-    keys: Chunk<A, NodeSize>,
-    children: Chunk<Option<PoolRef<Node<A>>>, Add1<NodeSize>>,
+    keys: Chunk<A, NODE_SIZE>,
+    children: Chunk<Option<PoolRef<Node<A>>>, NUM_CHILDREN>,
 }
 
 #[cfg(feature = "pool")]
-#[allow(unsafe_code)]
 unsafe fn cast_uninit<A>(target: &mut A) -> &mut mem::MaybeUninit<A> {
     &mut *(target as *mut A as *mut mem::MaybeUninit<A>)
 }
 
-#[allow(unsafe_code)]
 impl<A> PoolDefault for Node<A> {
     #[cfg(feature = "pool")]
     unsafe fn default_uninit(target: &mut mem::MaybeUninit<Self>) {
@@ -66,7 +63,6 @@ impl<A> PoolDefault for Node<A> {
     }
 }
 
-#[allow(unsafe_code)]
 impl<A> PoolClone for Node<A>
 where
     A: Clone,
@@ -1032,6 +1028,17 @@ pub struct Iter<'a, A> {
     pub(crate) remaining: usize,
 }
 
+// We impl Clone instead of deriving it, because we want Clone even if K and V aren't.
+impl<'a, A> Clone for Iter<'a, A> {
+    fn clone(&self) -> Self {
+        Iter {
+            fwd_path: self.fwd_path.clone(),
+            back_path: self.back_path.clone(),
+            remaining: self.remaining,
+        }
+    }
+}
+
 impl<'a, A: BTreeValue> Iter<'a, A> {
     pub(crate) fn new<R, BK>(root: &'a Node<A>, size: usize, range: R) -> Self
     where
@@ -1348,22 +1355,22 @@ impl<A: BTreeValue + Clone> ExactSizeIterator for ConsumingIter<A> {}
 // DiffIter
 
 /// An iterator over the differences between two ordered sets.
-pub struct DiffIter<'a, A> {
+pub struct DiffIter<'a, 'b, A> {
     old_stack: Vec<IterItem<'a, A>>,
-    new_stack: Vec<IterItem<'a, A>>,
+    new_stack: Vec<IterItem<'b, A>>,
 }
 
 /// A description of a difference between two ordered sets.
 #[derive(PartialEq, Eq, Debug)]
-pub enum DiffItem<'a, A> {
+pub enum DiffItem<'a, 'b, A> {
     /// This value has been added to the new set.
-    Add(&'a A),
+    Add(&'b A),
     /// This value has been changed between the two sets.
     Update {
         /// The old value.
         old: &'a A,
         /// The new value.
-        new: &'a A,
+        new: &'b A,
     },
     /// This value has been removed from the new set.
     Remove(&'a A),
@@ -1374,8 +1381,8 @@ enum IterItem<'a, A> {
     Yield(&'a A),
 }
 
-impl<'a, A: 'a> DiffIter<'a, A> {
-    pub(crate) fn new(old: &'a Node<A>, new: &'a Node<A>) -> Self {
+impl<'a, 'b, A: 'a + 'b> DiffIter<'a, 'b, A> {
+    pub(crate) fn new(old: &'a Node<A>, new: &'b Node<A>) -> Self {
         DiffIter {
             old_stack: if old.keys.is_empty() {
                 Vec::new()
@@ -1390,13 +1397,16 @@ impl<'a, A: 'a> DiffIter<'a, A> {
         }
     }
 
-    fn push_node(stack: &mut Vec<IterItem<'a, A>>, maybe_node: &'a Option<PoolRef<Node<A>>>) {
+    fn push_node<'either>(
+        stack: &mut Vec<IterItem<'either, A>>,
+        maybe_node: &'either Option<PoolRef<Node<A>>>,
+    ) {
         if let Some(node) = maybe_node {
             stack.push(IterItem::Consider(node))
         }
     }
 
-    fn push(stack: &mut Vec<IterItem<'a, A>>, node: &'a Node<A>) {
+    fn push<'either>(stack: &mut Vec<IterItem<'either, A>>, node: &'either Node<A>) {
         for n in 0..node.keys.len() {
             let i = node.keys.len() - n;
             Self::push_node(stack, &node.children[i]);
@@ -1406,11 +1416,11 @@ impl<'a, A: 'a> DiffIter<'a, A> {
     }
 }
 
-impl<'a, A> Iterator for DiffIter<'a, A>
+impl<'a, 'b, A> Iterator for DiffIter<'a, 'b, A>
 where
-    A: 'a + BTreeValue + PartialEq,
+    A: 'a + 'b + BTreeValue + PartialEq,
 {
-    type Item = DiffItem<'a, A>;
+    type Item = DiffItem<'a, 'b, A>;
 
     fn next(&mut self) -> Option<Self::Item> {
         loop {
