@@ -399,7 +399,7 @@ where
     /// the two maps, minus the number of elements belonging to nodes
     /// shared between them)
     #[must_use]
-    pub fn diff<'a>(&'a self, other: &'a Self) -> DiffIter<'a, K, V> {
+    pub fn diff<'a, 'b>(&'a self, other: &'b Self) -> DiffIter<'a, 'b, K, V> {
         DiffIter {
             it: NodeDiffIter::new(&self.root, &other.root),
         }
@@ -981,16 +981,20 @@ where
     /// ```
     #[inline]
     #[must_use]
-    pub fn union(self, other: Self) -> Self {
-        let (mut to_mutate, to_consume) = if self.len() >= other.len() {
-            (self, other)
+    pub fn union(mut self, mut other: Self) -> Self {
+        // We get better performance by consuming the small one and growing the big one. But the
+        // code isn't quite symmetric, because we need to keep values that are present in `self`.
+        if self.len() >= other.len() {
+            for (k, v) in other {
+                self.entry(k).or_insert(v);
+            }
+            self
         } else {
-            (other, self)
-        };
-        for (k, v) in to_consume {
-            to_mutate.entry(k).or_insert(v);
+            for (k, v) in self {
+                other.insert(k, v);
+            }
+            other
         }
-        to_mutate
     }
 
     /// Construct the union of two maps, using a function to decide
@@ -1553,6 +1557,7 @@ where
 
     /// Call the provided function to modify the value if the value
     /// exists.
+    #[must_use]
     pub fn and_modify<F>(mut self, f: F) -> Self
     where
         F: FnOnce(&mut V),
@@ -1853,6 +1858,15 @@ pub struct Iter<'a, K, V> {
     it: RangedIter<'a, (K, V)>,
 }
 
+// We impl Clone instead of deriving it, because we want Clone even if K and V aren't.
+impl<'a, K, V> Clone for Iter<'a, K, V> {
+    fn clone(&self) -> Self {
+        Iter {
+            it: self.it.clone(),
+        }
+    }
+}
+
 impl<'a, K, V> Iterator for Iter<'a, K, V>
 where
     (K, V): 'a + BTreeValue,
@@ -1880,31 +1894,31 @@ where
 impl<'a, K, V> ExactSizeIterator for Iter<'a, K, V> where (K, V): 'a + BTreeValue {}
 
 /// An iterator over the differences between two maps.
-pub struct DiffIter<'a, K, V> {
-    it: NodeDiffIter<'a, (K, V)>,
+pub struct DiffIter<'a, 'b, K, V> {
+    it: NodeDiffIter<'a, 'b, (K, V)>,
 }
 
 /// A description of a difference between two ordered maps.
 #[derive(PartialEq, Eq, Debug)]
-pub enum DiffItem<'a, K, V> {
+pub enum DiffItem<'a, 'b, K, V> {
     /// This value has been added to the new map.
-    Add(&'a K, &'a V),
+    Add(&'b K, &'b V),
     /// This value has been changed between the two maps.
     Update {
         /// The old value.
         old: (&'a K, &'a V),
         /// The new value.
-        new: (&'a K, &'a V),
+        new: (&'b K, &'b V),
     },
     /// This value has been removed from the new map.
     Remove(&'a K, &'a V),
 }
 
-impl<'a, K, V> Iterator for DiffIter<'a, K, V>
+impl<'a, 'b, K, V> Iterator for DiffIter<'a, 'b, K, V>
 where
-    (K, V): 'a + BTreeValue + PartialEq,
+    (K, V): 'a + 'b + BTreeValue + PartialEq,
 {
-    type Item = DiffItem<'a, K, V>;
+    type Item = DiffItem<'a, 'b, K, V>;
 
     fn next(&mut self) -> Option<Self::Item> {
         self.it.next().map(|item| match item {
@@ -2195,6 +2209,13 @@ mod test {
     use crate::test::is_sorted;
     use ::proptest::num::{i16, usize};
     use ::proptest::{bool, collection, proptest};
+    use static_assertions::{assert_impl_all, assert_not_impl_any};
+
+    assert_impl_all!(OrdMap<i32, i32>: Send, Sync);
+    assert_not_impl_any!(OrdMap<i32, *const i32>: Send, Sync);
+    assert_not_impl_any!(OrdMap<*const i32, i32>: Send, Sync);
+    assert_covariant!(OrdMap<T, i32> in T);
+    assert_covariant!(OrdMap<i32, T> in T);
 
     #[test]
     fn iterates_in_order() {
@@ -2343,15 +2364,15 @@ mod test {
     #[test]
     fn entry_api() {
         let mut map = ordmap! {"bar" => 5};
-        map.entry(&"foo").and_modify(|v| *v += 5).or_insert(1);
+        map.entry("foo").and_modify(|v| *v += 5).or_insert(1);
         assert_eq!(1, map[&"foo"]);
-        map.entry(&"foo").and_modify(|v| *v += 5).or_insert(1);
+        map.entry("foo").and_modify(|v| *v += 5).or_insert(1);
         assert_eq!(6, map[&"foo"]);
-        map.entry(&"bar").and_modify(|v| *v += 5).or_insert(1);
+        map.entry("bar").and_modify(|v| *v += 5).or_insert(1);
         assert_eq!(10, map[&"bar"]);
         assert_eq!(
             10,
-            match map.entry(&"bar") {
+            match map.entry("bar") {
                 Entry::Occupied(entry) => entry.remove(),
                 _ => panic!(),
             }
@@ -2420,6 +2441,8 @@ mod test {
                 omap.range((Included(i), Included(i + 7))).count(),
                 bmap.range((Included(i), Included(i + 7))).count(),
             );
+            assert_eq!(omap.range(..=i).next_back(), omap.get_prev(&i));
+            assert_eq!(omap.range(i..).next(), omap.get_next(&i));
         }
     }
 
@@ -2428,10 +2451,10 @@ mod test {
         let mut map = OrdMap::new();
         let contents = include_str!("test-fixtures/issue_124.txt");
         for line in contents.lines() {
-            if line.starts_with("insert ") {
-                map.insert(line[7..].parse::<u32>().unwrap(), 0);
-            } else if line.starts_with("remove ") {
-                map.remove(&line[7..].parse::<u32>().unwrap());
+            if let Some(tail) = line.strip_prefix("insert ") {
+                map.insert(tail.parse::<u32>().unwrap(), 0);
+            } else if let Some(tail) = line.strip_prefix("remove ") {
+                map.remove(&tail.parse::<u32>().unwrap());
             }
         }
     }
@@ -2631,9 +2654,9 @@ mod test {
             let diff: Vec<_> = a.diff(&b).collect();
             let union = b.clone().union(a.clone());
             let expected: Vec<_> = union.iter().filter_map(|(k, v)| {
-                if a.contains_key(&k) {
-                    if b.contains_key(&k) {
-                        let old = a.get(&k).unwrap();
+                if a.contains_key(k) {
+                    if b.contains_key(k) {
+                        let old = a.get(k).unwrap();
                         if old != v	{
                             Some(DiffItem::Update {
                                 old: (k, old),
@@ -2651,5 +2674,23 @@ mod test {
             }).collect();
             assert_eq!(expected, diff);
         }
+
+        #[test]
+        fn union(ref map1 in ord_map(i16::ANY, i16::ANY, 0..100),
+                 ref map2 in ord_map(i16::ANY, i16::ANY, 0..100)) {
+            let union_map = map1.clone().union(map2.clone());
+
+            for k in map1.keys() {
+                assert!(union_map.contains_key(k));
+            }
+
+            for k in map2.keys() {
+                assert!(union_map.contains_key(k));
+            }
+
+            for (k, v) in union_map.iter() {
+                assert_eq!(v, map1.get(k).or_else(|| map2.get(k)).unwrap());
+            }
+        }
     }
 }
