@@ -363,7 +363,7 @@ where
     /// the two sets, minus the number of elements belonging to nodes
     /// shared between them)
     #[must_use]
-    pub fn diff<'a>(&'a self, other: &'a Self) -> DiffIter<'_, A> {
+    pub fn diff<'a, 'b>(&'a self, other: &'b Self) -> DiffIter<'a, 'b, A> {
         DiffIter {
             it: NodeDiffIter::new(&self.root, &other.root),
         }
@@ -943,6 +943,15 @@ pub struct Iter<'a, A> {
     it: NodeIter<'a, Value<A>>,
 }
 
+// We impl Clone instead of deriving it, because we want Clone even if K and V aren't.
+impl<'a, A> Clone for Iter<'a, A> {
+    fn clone(&self) -> Self {
+        Iter {
+            it: self.it.clone(),
+        }
+    }
+}
+
 impl<'a, A> Iterator for Iter<'a, A>
 where
     A: 'a + Ord,
@@ -1028,15 +1037,15 @@ where
 }
 
 /// An iterator over the difference between two sets.
-pub struct DiffIter<'a, A> {
-    it: NodeDiffIter<'a, Value<A>>,
+pub struct DiffIter<'a, 'b, A> {
+    it: NodeDiffIter<'a, 'b, Value<A>>,
 }
 
-impl<'a, A> Iterator for DiffIter<'a, A>
+impl<'a, 'b, A> Iterator for DiffIter<'a, 'b, A>
 where
     A: Ord + PartialEq,
 {
-    type Item = DiffItem<'a, A>;
+    type Item = DiffItem<'a, 'b, A>;
 
     /// Advance the iterator and return the next value.
     ///
@@ -1180,6 +1189,11 @@ mod test {
     use super::*;
     use crate::proptest::*;
     use ::proptest::proptest;
+    use static_assertions::{assert_impl_all, assert_not_impl_any};
+
+    assert_impl_all!(OrdSet<i32>: Send, Sync);
+    assert_not_impl_any!(OrdSet<*const i32>: Send, Sync);
+    assert_covariant!(OrdSet<T> in T);
 
     #[test]
     fn match_strings_with_string_slices() {
