@@ -85,14 +85,16 @@ use crate::vector::{
 /// [split_at]: #method.split_at
 pub enum Focus<'a, A> {
     #[doc(hidden)]
+    /// The Single variant is a focus of a simple Vector that can be represented as a single slice.
     Single(&'a [A]),
     #[doc(hidden)]
+    /// The Full variant is a focus of a more complex Vector that cannot be represented as a single slice.
     Full(TreeFocus<A>),
 }
 
 impl<'a, A> Focus<'a, A>
 where
-    A: Clone + 'a,
+    A: 'a,
 {
     /// Construct a `Focus` for a [`Vector`][Vector].
     ///
@@ -173,6 +175,7 @@ where
     ///
     /// [slice::split_at]: https://doc.rust-lang.org/std/primitive.slice.html#method.split_at
     /// [Vector::split_at]: enum.Vector.html#method.split_at
+    #[must_use]
     pub fn narrow<R>(self, range: R) -> Self
     where
         R: RangeBounds<usize>,
@@ -257,9 +260,18 @@ where
 }
 
 pub struct TreeFocus<A> {
+    /// A clone of the Vector's internal tree that this focus points to. A clone ensures that we don't require a
+    /// reference to the original tree.
     tree: RRB<A>,
+    /// The view represents the range of the tree that this TreeFocus can see. The view can be narrowed by calling
+    /// either the narrow or split_at methods.
     view: Range<usize>,
+    /// The tree version of the Vector is represented as the concatenation of 2 chunks, followed by the tree root,
+    /// followed by 2 chunks. The middle_range refers to the range of the Vector that the tree covers.
     middle_range: Range<usize>,
+    /// This implementation of a focus stores only a single chunk for the Vector. This chunk can refer to one of the 4
+    /// chunks front/back chunks or one of the leaves of the tree. The target_ptr is the pointer to the actual chunk
+    /// in question. The target_range is the range that the chunk represents.
     target_range: Range<usize>,
     target_ptr: *const Chunk<A>,
 }
@@ -277,11 +289,7 @@ impl<A> Clone for TreeFocus<A> {
     }
 }
 
-#[allow(unsafe_code)]
-#[cfg(threadsafe)]
 unsafe impl<A: Send> Send for TreeFocus<A> {}
-#[allow(unsafe_code)]
-#[cfg(threadsafe)]
 unsafe impl<A: Sync> Sync for TreeFocus<A> {}
 
 #[inline]
@@ -289,10 +297,8 @@ fn contains<A: Ord>(range: &Range<A>, index: &A) -> bool {
     *index >= range.start && *index < range.end
 }
 
-impl<A> TreeFocus<A>
-where
-    A: Clone,
-{
+impl<A> TreeFocus<A> {
+    /// Creates a new TreeFocus for a Vector's RRB tree.
     fn new(tree: &RRB<A>) -> Self {
         let middle_start = tree.outer_f.len() + tree.inner_f.len();
         let middle_end = middle_start + tree.middle.len();
@@ -305,10 +311,12 @@ where
         }
     }
 
+    /// Returns the number of elements that the TreeFocus is valid for.
     fn len(&self) -> usize {
         self.view.end - self.view.start
     }
 
+    /// Restricts the TreeFocus to a subrange of itself.
     fn narrow(self, mut view: Range<usize>) -> Self {
         view.start += self.view.start;
         view.end += self.view.start;
@@ -321,6 +329,8 @@ where
         }
     }
 
+    /// Splits the TreeFocus into two disjoint foci. The first TreeFocus is valid for ..index while the
+    /// second is valid for index.. .
     fn split_at(self, index: usize) -> (Self, Self) {
         let len = self.len();
         let left = self.clone().narrow(0..index);
@@ -328,15 +338,18 @@ where
         (left, right)
     }
 
+    /// Computes an absolute index in the RRBTree for the given index relative to the start of this TreeFocus.
     fn physical_index(&self, index: usize) -> usize {
         debug_assert!(index < self.view.end);
         self.view.start + index
     }
 
+    /// Computes a range relative to the TreeFocus given one that is absolute in the RRBTree.
     fn logical_range(&self, range: &Range<usize>) -> Range<usize> {
         (range.start - self.view.start)..(range.end - self.view.start)
     }
 
+    /// Sets the internal chunk to the one that contains the given absolute index.
     fn set_focus(&mut self, index: usize) {
         if index < self.middle_range.start {
             let outer_len = self.tree.outer_f.len();
@@ -368,11 +381,12 @@ where
         }
     }
 
-    #[allow(unsafe_code)]
+    /// Gets the chunk that this TreeFocus is focused on.
     fn get_focus(&self) -> &Chunk<A> {
         unsafe { &*self.target_ptr }
     }
 
+    /// Gets the value at the given index relative to the TreeFocus.
     pub fn get(&mut self, index: usize) -> Option<&A> {
         if index >= self.len() {
             return None;
@@ -385,6 +399,7 @@ where
         Some(&self.get_focus()[target_phys_index])
     }
 
+    /// Gets the chunk for an index as a slice and its corresponding range within the TreeFocus.
     pub fn get_chunk(&mut self, index: usize) -> (Range<usize>, &[A]) {
         let phys_index = self.physical_index(index);
         if !contains(&self.target_range, &phys_index) {
@@ -462,27 +477,17 @@ where
 /// [Focus]: enum.Focus.html
 pub enum FocusMut<'a, A> {
     #[doc(hidden)]
+    /// The Single variant is a focusmut of a simple Vector that can be represented as a single slice.
     Single(RRBPool<A>, &'a mut [A]),
     #[doc(hidden)]
+    /// The Full variant is a focus of a more complex Vector that cannot be represented as a single slice.
     Full(RRBPool<A>, TreeFocusMut<'a, A>),
 }
 
 impl<'a, A> FocusMut<'a, A>
 where
-    A: Clone + 'a,
+    A: 'a,
 {
-    /// Construct a `FocusMut` for a `Vector`.
-    pub fn new(vector: &'a mut Vector<A>) -> Self {
-        match &mut vector.vector {
-            Inline(pool, chunk) => FocusMut::Single(pool.clone(), chunk),
-            Single(pool, chunk) => FocusMut::Single(
-                pool.clone(),
-                PoolRef::make_mut(&pool.value_pool, chunk).as_mut_slice(),
-            ),
-            Full(pool, tree) => FocusMut::Full(pool.clone(), TreeFocusMut::new(tree)),
-        }
-    }
-
     /// Get the length of the focused `Vector`.
     pub fn len(&self) -> usize {
         match self {
@@ -496,133 +501,6 @@ where
         self.len() == 0
     }
 
-    /// Get a reference to the value at a given index.
-    pub fn get(&mut self, index: usize) -> Option<&A> {
-        self.get_mut(index).map(|r| &*r)
-    }
-
-    /// Get a mutable reference to the value at a given index.
-    pub fn get_mut(&mut self, index: usize) -> Option<&mut A> {
-        match self {
-            FocusMut::Single(_, chunk) => chunk.get_mut(index),
-            FocusMut::Full(pool, tree) => tree.get(pool, index),
-        }
-    }
-
-    /// Get a reference to the value at a given index.
-    ///
-    /// Panics if the index is out of bounds.
-    pub fn index(&mut self, index: usize) -> &A {
-        &*self.index_mut(index)
-    }
-
-    /// Get a mutable reference to the value at a given index.
-    ///
-    /// Panics if the index is out of bounds.
-    #[allow(clippy::should_implement_trait)] // would if I could
-    pub fn index_mut(&mut self, index: usize) -> &mut A {
-        self.get_mut(index).expect("index out of bounds")
-    }
-
-    /// Update the value at a given index.
-    ///
-    /// Returns `None` if the index is out of bounds, or the replaced value
-    /// otherwise.
-    pub fn set(&mut self, index: usize, value: A) -> Option<A> {
-        self.get_mut(index).map(|pos| replace(pos, value))
-    }
-
-    /// Swap the values at two given indices.
-    ///
-    /// Panics if either index is out of bounds.
-    ///
-    /// If the indices are equal, this function returns without doing anything.
-    pub fn swap(&mut self, a: usize, b: usize) {
-        if a == b {
-            return;
-        }
-        self.pair(a, b, |left, right| swap(left, right));
-    }
-
-    /// Lookup two indices simultaneously and run a function over them.
-    ///
-    /// Useful because the borrow checker won't let you have more than one
-    /// mutable reference into the same data structure at any given time.
-    ///
-    /// Panics if either index is out of bounds, or if they are the same index.
-    ///
-    /// # Examples
-    ///
-    /// ```rust
-    /// # #[macro_use] extern crate imbl;
-    /// # use imbl::vector::Vector;
-    /// # use std::iter::FromIterator;
-    /// let mut vec = vector![1, 2, 3, 4, 5];
-    /// vec.focus_mut().pair(1, 3, |a, b| *a += *b);
-    /// assert_eq!(vector![1, 6, 3, 4, 5], vec);
-    /// ```
-    #[allow(unsafe_code)]
-    pub fn pair<F, B>(&mut self, a: usize, b: usize, mut f: F) -> B
-    where
-        F: FnMut(&mut A, &mut A) -> B,
-    {
-        if a == b {
-            panic!("vector::FocusMut::pair: indices cannot be equal!");
-        }
-        let pa: *mut A = self.index_mut(a);
-        let pb: *mut A = self.index_mut(b);
-        unsafe { f(&mut *pa, &mut *pb) }
-    }
-
-    /// Lookup three indices simultaneously and run a function over them.
-    ///
-    /// Useful because the borrow checker won't let you have more than one
-    /// mutable reference into the same data structure at any given time.
-    ///
-    /// Panics if any index is out of bounds, or if any indices are equal.
-    ///
-    /// # Examples
-    ///
-    /// ```rust
-    /// # #[macro_use] extern crate imbl;
-    /// # use imbl::vector::Vector;
-    /// # use std::iter::FromIterator;
-    /// let mut vec = vector![1, 2, 3, 4, 5];
-    /// vec.focus_mut().triplet(0, 2, 4, |a, b, c| *a += *b + *c);
-    /// assert_eq!(vector![9, 2, 3, 4, 5], vec);
-    /// ```
-    #[allow(unsafe_code)]
-    pub fn triplet<F, B>(&mut self, a: usize, b: usize, c: usize, mut f: F) -> B
-    where
-        F: FnMut(&mut A, &mut A, &mut A) -> B,
-    {
-        if a == b || b == c || a == c {
-            panic!("vector::FocusMut::triplet: indices cannot be equal!");
-        }
-        let pa: *mut A = self.index_mut(a);
-        let pb: *mut A = self.index_mut(b);
-        let pc: *mut A = self.index_mut(c);
-        unsafe { f(&mut *pa, &mut *pb, &mut *pc) }
-    }
-
-    /// Get the chunk for the given index.
-    ///
-    /// This gives you a reference to the leaf node that contains the index,
-    /// along with its start and end indices.
-    pub fn chunk_at(&mut self, index: usize) -> (Range<usize>, &mut [A]) {
-        let len = self.len();
-        if index >= len {
-            panic!("vector::FocusMut::chunk_at: index out of bounds");
-        }
-        match self {
-            FocusMut::Single(_, chunk) => (0..len, chunk),
-            FocusMut::Full(pool, tree) => {
-                let (range, chunk) = tree.get_chunk(pool, index);
-                (range, chunk)
-            }
-        }
-    }
-
     /// Narrow the focus onto a subslice of the vector.
     ///
     /// `FocusMut::narrow(range)` has the same effect as `&slice[range]`, without
@@ -644,6 +522,7 @@ where
     ///
     /// [slice::split_at]: https://doc.rust-lang.org/std/primitive.slice.html#method.split_at
     /// [Vector::split_at]: enum.Vector.html#method.split_at
+    #[must_use]
     pub fn narrow<R>(self, range: R) -> Self
     where
         R: RangeBounds<usize>,
@@ -734,6 +613,148 @@ where
     }
 }
 
+impl<'a, A> FocusMut<'a, A>
+where
+    A: Clone + 'a,
+{
+    /// Construct a `FocusMut` for a `Vector`.
+    pub fn new(vector: &'a mut Vector<A>) -> Self {
+        match &mut vector.vector {
+            Inline(pool, chunk) => FocusMut::Single(pool.clone(), chunk),
+            Single(pool, chunk) => FocusMut::Single(
+                pool.clone(),
+                PoolRef::make_mut(&pool.value_pool, chunk).as_mut_slice(),
+            ),
+            Full(pool, tree) => FocusMut::Full(pool.clone(), TreeFocusMut::new(tree)),
+        }
+    }
+
+    /// Get a reference to the value at a given index.
+    pub fn get(&mut self, index: usize) -> Option<&A> {
+        self.get_mut(index).map(|r| &*r)
+    }
+
+    /// Get a mutable reference to the value at a given index.
+    pub fn get_mut(&mut self, index: usize) -> Option<&mut A> {
+        match self {
+            FocusMut::Single(_, chunk) => chunk.get_mut(index),
+            FocusMut::Full(pool, tree) => tree.get(pool, index),
+        }
+    }
+
+    /// Get a reference to the value at a given index.
+    ///
+    /// Panics if the index is out of bounds.
+    pub fn index(&mut self, index: usize) -> &A {
+        &*self.index_mut(index)
+    }
+
+    /// Get a mutable reference to the value at a given index.
+    ///
+    /// Panics if the index is out of bounds.
+    #[allow(clippy::should_implement_trait)] // would if I could
+    pub fn index_mut(&mut self, index: usize) -> &mut A {
+        self.get_mut(index).expect("index out of bounds")
+    }
+
+    /// Update the value at a given index.
+    ///
+    /// Returns `None` if the index is out of bounds, or the replaced value
+    /// otherwise.
+    pub fn set(&mut self, index: usize, value: A) -> Option<A> {
+        self.get_mut(index).map(|pos| replace(pos, value))
+    }
+
+    /// Swap the values at two given indices.
+    ///
+    /// Panics if either index is out of bounds.
+    ///
+    /// If the indices are equal, this function returns without doing anything.
+    pub fn swap(&mut self, a: usize, b: usize) {
+        if a == b {
+            return;
+        }
+        self.pair(a, b, |left, right| swap(left, right));
+    }
+
+    /// Lookup two indices simultaneously and run a function over them.
+    ///
+    /// Useful because the borrow checker won't let you have more than one
+    /// mutable reference into the same data structure at any given time.
+    ///
+    /// Panics if either index is out of bounds, or if they are the same index.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
+    /// # use std::iter::FromIterator;
+    /// let mut vec = vector![1, 2, 3, 4, 5];
+    /// vec.focus_mut().pair(1, 3, |a, b| *a += *b);
+    /// assert_eq!(vector![1, 6, 3, 4, 5], vec);
+    /// ```
+    pub fn pair<F, B>(&mut self, a: usize, b: usize, mut f: F) -> B
+    where
+        F: FnMut(&mut A, &mut A) -> B,
+    {
+        if a == b {
+            panic!("vector::FocusMut::pair: indices cannot be equal!");
+        }
+        let pa: *mut A = self.index_mut(a);
+        let pb: *mut A = self.index_mut(b);
+        unsafe { f(&mut *pa, &mut *pb) }
+    }
+
+    /// Lookup three indices simultaneously and run a function over them.
+    ///
+    /// Useful because the borrow checker won't let you have more than one
+    /// mutable reference into the same data structure at any given time.
+    ///
+    /// Panics if any index is out of bounds, or if any indices are equal.
+    ///
+    /// # Examples
+    ///
+    /// ```rust
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::vector::Vector;
+    /// # use std::iter::FromIterator;
+    /// let mut vec = vector![1, 2, 3, 4, 5];
+    /// vec.focus_mut().triplet(0, 2, 4, |a, b, c| *a += *b + *c);
+    /// assert_eq!(vector![9, 2, 3, 4, 5], vec);
+    /// ```
+    pub fn triplet<F, B>(&mut self, a: usize, b: usize, c: usize, mut f: F) -> B
+    where
+        F: FnMut(&mut A, &mut A, &mut A) -> B,
+    {
+        if a == b || b == c || a == c {
+            panic!("vector::FocusMut::triplet: indices cannot be equal!");
+        }
+        let pa: *mut A = self.index_mut(a);
+        let pb: *mut A = self.index_mut(b);
+        let pc: *mut A = self.index_mut(c);
+        unsafe { f(&mut *pa, &mut *pb, &mut *pc) }
+    }
+
+    /// Get the chunk for the given index.
+    ///
+    /// This gives you a reference to the leaf node that contains the index,
+    /// along with its start and end indices.
+    pub fn chunk_at(&mut self, index: usize) -> (Range<usize>, &mut [A]) {
+        let len = self.len();
+        if index >= len {
+            panic!("vector::FocusMut::chunk_at: index out of bounds");
+        }
+        match self {
+            FocusMut::Single(_, chunk) => (0..len, chunk),
+            FocusMut::Full(pool, tree) => {
+                let (range, chunk) = tree.get_chunk(pool, index);
+                (range, chunk)
+            }
+        }
+    }
+}
+
 impl<'a, A> IntoIterator for FocusMut<'a, A>
 where
     A: Clone + 'a,
@@ -755,18 +776,33 @@ where
     }
 }
 
+// NOTE: The documentation the mutable version is similar to the non-mutable version. I will comment for the places
+// where there are differences, otherwise the documentation is copied directly.
 pub struct TreeFocusMut<'a, A> {
+    /// The tree that this TreeFocusMut refers to. Unlike the non-mutable version, TreeFocusMut needs to store a
+    /// mutable reference. Additionally, there may be multiple TreeFocusMuts that refer to the same tree so we need a
+    /// Lock to synchronise the changes.
     tree: Lock<&'a mut RRB<A>>,
+    /// The view represents the range of the tree that this TreeFocusMut can see. The view can be narrowed by calling
+    /// either the narrow or split_at methods.
     view: Range<usize>,
+    /// The tree version of the Vector is represented as the concatenation of 2 chunks, followed by the tree root,
+    /// followed by 2 chunks. The middle_range refers to the range of the Vector that the tree covers.
     middle_range: Range<usize>,
+    /// This implementation of a focusmut stores only a single chunk for the Vector. This chunk can refer to one of the
+    /// 4 chunks front/back chunks or one of the leaves of the tree. The target_ptr is the pointer to the actual chunk
+    /// in question. The target_range is the range that the chunk represents.
     target_range: Range<usize>,
+    /// Not actually sure why this needs to be an atomic, it seems like it is unneccessary. This is just a pointer to
+    /// the chunk referred to above.
     target_ptr: AtomicPtr<Chunk<A>>,
 }
 
 impl<'a, A> TreeFocusMut<'a, A>
 where
-    A: Clone + 'a,
+    A: 'a,
 {
+    /// Creates a new TreeFocusMut for a Vector's RRB tree.
     fn new(tree: &'a mut RRB<A>) -> Self {
         let middle_start = tree.outer_f.len() + tree.inner_f.len();
         let middle_end = middle_start + tree.middle.len();
@@ -779,10 +815,12 @@ where
         }
     }
 
+    /// Returns the number of elements that the TreeFocusMut is valid for.
     fn len(&self) -> usize {
         self.view.end - self.view.start
     }
 
+    /// Restricts the TreeFocusMut to a subrange of itself.
     fn narrow(self, mut view: Range<usize>) -> Self {
         view.start += self.view.start;
         view.end += self.view.start;
@@ -795,10 +833,11 @@ where
         }
     }
 
+    /// Splits the TreeFocusMut into two disjoint foci. The first TreeFocusMut is valid for ..index while the
+    /// second is valid for index.. .
     fn split_at(self, index: usize) -> (Self, Self) {
         let len = self.len();
         debug_assert!(index <= len);
-        #[allow(unsafe_code)]
         let left = TreeFocusMut {
             view: self.view.start..(self.view.start + index),
             middle_range: self.middle_range.clone(),
@@ -816,15 +855,28 @@ where
         (left, right)
     }
 
+    /// Computes an absolute index in the RRBTree for the given index relative to the start of this TreeFocusMut.
     fn physical_index(&self, index: usize) -> usize {
         debug_assert!(index < self.view.end);
         self.view.start + index
     }
 
+    /// Computes a range relative to the TreeFocusMut given one that is absolute in the RRBTree.
     fn logical_range(&self, range: &Range<usize>) -> Range<usize> {
         (range.start - self.view.start)..(range.end - self.view.start)
     }
 
+    /// Gets the chunk for an index and its corresponding range within the TreeFocusMut.
+    fn get_focus(&mut self) -> &mut Chunk<A> {
+        unsafe { &mut *self.target_ptr.load(Ordering::Relaxed) }
+    }
+}
+
+impl<'a, A> TreeFocusMut<'a, A>
+where
+    A: Clone + 'a,
+{
+    /// Sets the internal chunk to the one that contains the given absolute index.
     fn set_focus(&mut self, pool: &RRBPool<A>, index: usize) {
         let mut tree = self
             .tree
@@ -871,11 +923,7 @@ where
         }
     }
 
-    #[allow(unsafe_code)]
-    fn get_focus(&mut self) -> &mut Chunk<A> {
-        unsafe { &mut *self.target_ptr.load(Ordering::Relaxed) }
-    }
-
+    /// Gets the value at the given index relative to the TreeFocusMut.
     pub fn get(&mut self, pool: &RRBPool<A>, index: usize) -> Option<&mut A> {
         if index >= self.len() {
             return None;
@@ -888,6 +936,7 @@ where
         Some(&mut self.get_focus()[target_phys_index])
     }
 
+    /// Gets the chunk for an index as a slice and its corresponding range within the TreeFocusMut.
     pub fn get_chunk(&mut self, pool: &RRBPool<A>, index: usize) -> (Range<usize>, &mut [A]) {
         let phys_index = self.physical_index(index);
         if !contains(&self.target_range, &phys_index) {
