@@ -43,6 +43,8 @@
 //! [Vec]: https://doc.rust-lang.org/std/vec/struct.Vec.html
 //! [VecDeque]: https://doc.rust-lang.org/std/collections/struct.VecDeque.html
 
+#![allow(unsafe_code)]
+
 use std::borrow::Borrow;
 use std::cmp::Ordering;
 use std::fmt::{Debug, Error, Formatter};
@@ -52,12 +54,12 @@ use std::iter::{FromIterator, FusedIterator};
 use std::mem::{replace, swap};
 use std::ops::{Add, Index, IndexMut, RangeBounds};
 
-use sized_chunks::InlineArray;
+use imbl_sized_chunks::InlineArray;
 
 use crate::nodes::chunk::{Chunk, CHUNK_SIZE};
 use crate::nodes::rrb::{Node, PopResult, PushResult, SplitResult};
 use crate::sort;
-use crate::util::{clone_ref, swap_indices, to_range, Pool, PoolDefault, PoolRef, Ref, Side};
+use crate::util::{clone_ref, to_range, Pool, PoolDefault, PoolRef, Ref, Side};
 
 use self::VectorInner::{Full, Inline, Single};
 
@@ -68,7 +70,7 @@ pub use self::focus::{Focus, FocusMut};
 mod pool;
 pub use self::pool::RRBPool;
 
-#[cfg(all(threadsafe, any(test, feature = "rayon")))]
+#[cfg(any(test, feature = "rayon"))]
 pub mod rayon;
 
 /// Construct a vector from a sequence of elements.
@@ -177,7 +179,7 @@ impl<A> Clone for RRB<A> {
     }
 }
 
-impl<A: Clone> Vector<A> {
+impl<A> Vector<A> {
     /// Get a reference to the memory pool this `Vector` is using.
     ///
     /// Note that if you didn't specifically construct it with a pool, you'll
@@ -195,8 +197,12 @@ impl<A: Clone> Vector<A> {
     /// to grow further.
     fn needs_promotion(&self) -> bool {
         match &self.vector {
-            Inline(_, chunk) if chunk.is_full() => true,
-            Single(_, chunk) if chunk.is_full() => true,
+            // Prevent the inline array from getting bigger than a single chunk. This means that we
+            // can always promote `Inline` to `Single`, even when we're configured to have a small
+            // chunk size. (TODO: it might be better to just never use `Single` in this situation,
+            // but that's a more invasive change.)
+            Inline(_, chunk) => chunk.is_full() || chunk.len() + 1 >= CHUNK_SIZE,
+            Single(_, chunk) => chunk.is_full(),
             _ => false,
         }
     }
@@ -384,15 +390,6 @@ impl<A: Clone> Vector<A> {
         Iter::new(self)
     }
 
-    /// Get a mutable iterator over a vector.
-    ///
-    /// Time: O(1)
-    #[inline]
-    #[must_use]
-    pub fn iter_mut(&mut self) -> IterMut<'_, A> {
-        IterMut::new(self)
-    }
-
     /// Get an iterator over the leaf nodes of a vector.
     ///
     /// This returns an iterator over the [`Chunk`s][Chunk] at the leaves of the
@@ -408,21 +405,6 @@ impl<A: Clone> Vector<A> {
         Chunks::new(self)
     }
 
-    /// Get a mutable iterator over the leaf nodes of a vector.
-    //
-    /// This returns an iterator over the [`Chunk`s][Chunk] at the leaves of the
-    /// RRB tree. These are useful for efficient parallelisation of work on
-    /// the vector, but should not be used for basic iteration.
-    ///
-    /// Time: O(1)
-    ///
-    /// [Chunk]: ../chunk/struct.Chunk.html
-    #[inline]
-    #[must_use]
-    pub fn leaves_mut(&mut self) -> ChunksMut<'_, A> {
-        ChunksMut::new(self)
-    }
-
     /// Construct a [`Focus`][Focus] for a vector.
     ///
     /// Time: O(1)
@@ -434,17 +416,6 @@ impl<A: Clone> Vector<A> {
         Focus::new(self)
     }
 
-    /// Construct a [`FocusMut`][FocusMut] for a vector.
-    ///
-    /// Time: O(1)
-    ///
-    /// [FocusMut]: enum.FocusMut.html
-    #[inline]
-    #[must_use]
-    pub fn focus_mut(&mut self) -> FocusMut<'_, A> {
-        FocusMut::new(self)
-    }
-
     /// Get a reference to the value at index `index` in a vector.
     ///
     /// Returns `None` if the index is out of bounds.
@@ -497,68 +468,6 @@ impl<A: Clone> Vector<A> {
         }
     }
 
-    /// Get a mutable reference to the value at index `index` in a
-    /// vector.
-    ///
-    /// Returns `None` if the index is out of bounds.
-    ///
-    /// Time: O(log n)
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// # #[macro_use] extern crate imbl;
-    /// # use imbl::Vector;
-    /// let mut vec = vector!["Joe", "Mike", "Robert"];
-    /// {
-    ///     let robert = vec.get_mut(2).unwrap();
-    ///     assert_eq!(&mut "Robert", robert);
-    ///     *robert = "Bjarne";
-    /// }
-    /// assert_eq!(vector!["Joe", "Mike", "Bjarne"], vec);
-    /// ```
-    #[must_use]
-    pub fn get_mut(&mut self, index: usize) -> Option<&mut A> {
-        if index >= self.len() {
-            return None;
-        }
-
-        match &mut self.vector {
-            Inline(_, chunk) => chunk.get_mut(index),
-            Single(pool, chunk) => PoolRef::make_mut(&pool.value_pool, chunk).get_mut(index),
-            Full(pool, tree) => {
-                let mut local_index = index;
-
-                if local_index < tree.outer_f.len() {
-                    let outer_f = PoolRef::make_mut(&pool.value_pool, &mut tree.outer_f);
-                    return Some(&mut outer_f[local_index]);
-                }
-                local_index -= tree.outer_f.len();
-
-                if local_index < tree.inner_f.len() {
-                    let inner_f = PoolRef::make_mut(&pool.value_pool, &mut tree.inner_f);
-                    return Some(&mut inner_f[local_index]);
-                }
-                local_index -= tree.inner_f.len();
-
-                if local_index < tree.middle.len() {
-                    let middle = Ref::make_mut(&mut tree.middle);
-                    return Some(middle.index_mut(pool, tree.middle_level, local_index));
-                }
-                local_index -= tree.middle.len();
-
-                if local_index < tree.inner_b.len() {
-                    let inner_b = PoolRef::make_mut(&pool.value_pool, &mut tree.inner_b);
-                    return Some(&mut inner_b[local_index]);
-                }
-                local_index -= tree.inner_b.len();
-
-                let outer_b = PoolRef::make_mut(&pool.value_pool, &mut tree.outer_b);
-                Some(&mut outer_b[local_index])
-            }
-        }
-    }
-
     /// Get the first element of a vector.
     ///
     /// If the vector is empty, `None` is returned.
@@ -570,17 +479,6 @@ impl<A: Clone> Vector<A> {
         self.get(0)
     }
 
-    /// Get a mutable reference to the first element of a vector.
-    ///
-    /// If the vector is empty, `None` is returned.
-    ///
-    /// Time: O(log n)
-    #[inline]
-    #[must_use]
-    pub fn front_mut(&mut self) -> Option<&mut A> {
-        self.get_mut(0)
-    }
-
     /// Get the first element of a vector.
     ///
     /// If the vector is empty, `None` is returned.
@@ -610,21 +508,6 @@ impl<A: Clone> Vector<A> {
         }
     }
 
-    /// Get a mutable reference to the last element of a vector.
-    ///
-    /// If the vector is empty, `None` is returned.
-    ///
-    /// Time: O(log n)
-    #[must_use]
-    pub fn back_mut(&mut self) -> Option<&mut A> {
-        if self.is_empty() {
-            None
-        } else {
-            let len = self.len();
-            self.get_mut(len - 1)
-        }
-    }
-
     /// Get the last element of a vector.
     ///
     /// If the vector is empty, `None` is returned.
@@ -673,7 +556,7 @@ impl<A: Clone> Vector<A> {
     /// Test if a given element is in the vector.
     ///
     /// Searches the vector for the first occurrence of a given value,
-    /// and returns `true if it's there. If it's nowhere to be found
+    /// and returns `true` if it's there. If it's nowhere to be found
     /// in the vector, it returns `false`.
     ///
     /// Time: O(n)
@@ -783,9 +666,7 @@ impl<A: Clone> Vector<A> {
     {
         self.binary_search_by(|k| f(k).cmp(b))
     }
-}
 
-impl<A: Clone> Vector<A> {
     /// Construct a vector with a single value.
     ///
     /// # Examples
@@ -818,6 +699,157 @@ impl<A: Clone> Vector<A> {
         }
     }
 
+    /// Dump the internal RRB tree into graphviz format.
+    ///
+    /// This method requires the `debug` feature flag.
+    #[cfg(any(test, feature = "debug"))]
+    pub fn dot<W: std::io::Write>(&self, write: W) -> std::io::Result<()> {
+        if let Full(_, ref tree) = self.vector {
+            tree.middle.dot(write)
+        } else {
+            Ok(())
+        }
+    }
+
+    /// Verify the internal consistency of a vector.
+    ///
+    /// This method walks the RRB tree making up the current `Vector`
+    /// (if it has one) and verifies that all the invariants hold.
+    /// If something is wrong, it will panic.
+    ///
+    /// This method requires the `debug` feature flag.
+    #[cfg(any(test, feature = "debug"))]
+    pub fn assert_invariants(&self) {
+        if let Full(_, ref tree) = self.vector {
+            tree.assert_invariants();
+        }
+    }
+}
+
+impl<A: Clone> Vector<A> {
+    /// Get a mutable reference to the value at index `index` in a
+    /// vector.
+    ///
+    /// Returns `None` if the index is out of bounds.
+    ///
+    /// Time: O(log n)
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # #[macro_use] extern crate imbl;
+    /// # use imbl::Vector;
+    /// let mut vec = vector!["Joe", "Mike", "Robert"];
+    /// {
+    ///     let robert = vec.get_mut(2).unwrap();
+    ///     assert_eq!(&mut "Robert", robert);
+    ///     *robert = "Bjarne";
+    /// }
+    /// assert_eq!(vector!["Joe", "Mike", "Bjarne"], vec);
+    /// ```
+    #[must_use]
+    pub fn get_mut(&mut self, index: usize) -> Option<&mut A> {
+        if index >= self.len() {
+            return None;
+        }
+
+        match &mut self.vector {
+            Inline(_, chunk) => chunk.get_mut(index),
+            Single(pool, chunk) => PoolRef::make_mut(&pool.value_pool, chunk).get_mut(index),
+            Full(pool, tree) => {
+                let mut local_index = index;
+
+                if local_index < tree.outer_f.len() {
+                    let outer_f = PoolRef::make_mut(&pool.value_pool, &mut tree.outer_f);
+                    return Some(&mut outer_f[local_index]);
+                }
+                local_index -= tree.outer_f.len();
+
+                if local_index < tree.inner_f.len() {
+                    let inner_f = PoolRef::make_mut(&pool.value_pool, &mut tree.inner_f);
+                    return Some(&mut inner_f[local_index]);
+                }
+                local_index -= tree.inner_f.len();
+
+                if local_index < tree.middle.len() {
+                    let middle = Ref::make_mut(&mut tree.middle);
+                    return Some(middle.index_mut(pool, tree.middle_level, local_index));
+                }
+                local_index -= tree.middle.len();
+
+                if local_index < tree.inner_b.len() {
+                    let inner_b = PoolRef::make_mut(&pool.value_pool, &mut tree.inner_b);
+                    return Some(&mut inner_b[local_index]);
+                }
+                local_index -= tree.inner_b.len();
+
+                let outer_b = PoolRef::make_mut(&pool.value_pool, &mut tree.outer_b);
+                Some(&mut outer_b[local_index])
+            }
+        }
+    }
+
+    /// Get a mutable reference to the first element of a vector.
+    ///
+    /// If the vector is empty, `None` is returned.
+    ///
+    /// Time: O(log n)
+    #[inline]
+    #[must_use]
+    pub fn front_mut(&mut self) -> Option<&mut A> {
+        self.get_mut(0)
+    }
+
+    /// Get a mutable reference to the last element of a vector.
+    ///
+    /// If the vector is empty, `None` is returned.
+    ///
+    /// Time: O(log n)
+    #[must_use]
+    pub fn back_mut(&mut self) -> Option<&mut A> {
+        if self.is_empty() {
+            None
+        } else {
+            let len = self.len();
+            self.get_mut(len - 1)
+        }
+    }
+
+    /// Construct a [`FocusMut`][FocusMut] for a vector.
+    ///
+    /// Time: O(1)
+    ///
+    /// [FocusMut]: enum.FocusMut.html
+    #[inline]
+    #[must_use]
+    pub fn focus_mut(&mut self) -> FocusMut<'_, A> {
+        FocusMut::new(self)
+    }
+
+    /// Get a mutable iterator over a vector.
+    ///
+    /// Time: O(1)
+    #[inline]
+    #[must_use]
+    pub fn iter_mut(&mut self) -> IterMut<'_, A> {
+        IterMut::new(self)
+    }
+
+    /// Get a mutable iterator over the leaf nodes of a vector.
+    //
+    /// This returns an iterator over the [`Chunk`s][Chunk] at the leaves of the
+    /// RRB tree. These are useful for efficient parallelisation of work on
+    /// the vector, but should not be used for basic iteration.
+    ///
+    /// Time: O(1)
+    ///
+    /// [Chunk]: ../chunk/struct.Chunk.html
+    #[inline]
+    #[must_use]
+    pub fn leaves_mut(&mut self) -> ChunksMut<'_, A> {
+        ChunksMut::new(self)
+    }
+
     /// Create a new vector with the value at index `index` updated.
     ///
     /// Panics if the index is out of bounds.
@@ -855,7 +887,16 @@ impl<A: Clone> Vector<A> {
     ///
     /// Time: O(log n)
     pub fn swap(&mut self, i: usize, j: usize) {
-        swap_indices(self, i, j)
+        if i != j {
+            let a: *mut A = &mut self[i];
+            let b: *mut A = &mut self[j];
+
+            // Vector's implementation of IndexMut ensures that if `i` and `j` are different
+            // indices then `&mut self[i]` and `&mut self[j]` are non-overlapping.
+            unsafe {
+                std::ptr::swap(a, b);
+            }
+        }
     }
 
     /// Push a value to the front of a vector.
@@ -1106,7 +1147,7 @@ impl<A: Clone> Vector<A> {
             }
         }
         if del > 0 {
-            self.split_off(len - del);
+            let _ = self.split_off(len - del);
         }
     }
 
@@ -1151,6 +1192,7 @@ impl<A: Clone> Vector<A> {
     /// assert_eq!(vector![1, 2, 3], left);
     /// assert_eq!(vector![7, 8, 9], right);
     /// ```
+    #[must_use]
     pub fn split_off(&mut self, index: usize) -> Self {
         assert!(index <= self.len());
 
@@ -1309,7 +1351,7 @@ impl<A: Clone> Vector<A> {
     pub fn take(&self, count: usize) -> Self {
         // FIXME can be made more efficient by dropping the unwanted side without constructing it
         let mut left = self.clone();
-        left.split_off(count);
+        let _ = left.split_off(count);
         left
     }
 
@@ -1322,7 +1364,7 @@ impl<A: Clone> Vector<A> {
     /// Time: O(log n)
     pub fn truncate(&mut self, len: usize) {
         // FIXME can be made more efficient by dropping the unwanted side without constructing it
-        self.split_off(len);
+        let _ = self.split_off(len);
     }
 
     /// Extract a slice from a vector.
@@ -1332,6 +1374,7 @@ impl<A: Clone> Vector<A> {
     /// vector.
     ///
     /// Time: O(log n)
+    #[must_use]
     pub fn slice<R>(&mut self, range: R) -> Self
     where
         R: RangeBounds<usize>,
@@ -1371,11 +1414,7 @@ impl<A: Clone> Vector<A> {
             return self.push_back(value);
         }
         assert!(index < self.len());
-        if if let Inline(_, chunk) = &self.vector {
-            chunk.is_full()
-        } else {
-            false
-        } {
+        if matches!(&self.vector, Inline(_, _)) && self.needs_promotion() {
             self.promote_inline();
         }
         match &mut self.vector {
@@ -1457,6 +1496,89 @@ impl<A: Clone> Vector<A> {
         }
     }
 
+    /// Insert an element into a sorted vector using a comparator function.
+    ///
+    /// Insert an element into a vector in sorted order using the given
+    /// comparator function, assuming the vector is already in sorted order.
+    ///
+    /// Note that the ordering used to sort the vector must logically match
+    /// the ordering in the comparison function provided to `insert_ord_by`.
+    /// Incompatible definitions of the ordering won't result in memory
+    /// unsafety, but will likely result in out-of-order insertions.
+    ///
+    ///
+    /// Time: O(log n)
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # #[macro_use] extern crate imbl;
+    /// use imbl::vector::Vector;
+    ///
+    /// let mut vec: Vector<u8> = vector![9, 8, 7, 3, 2, 1];
+    /// vec.insert_ord_by(5, |a, b| a.cmp(b).reverse());
+    /// assert_eq!(vector![9, 8, 7, 5, 3, 2, 1], vec);
+    ///
+    /// // Note that `insert_ord` does not work in this case because it uses
+    /// // the default comparison function for the item type.
+    /// vec.insert_ord(4);
+    /// assert_eq!(vector![4, 9, 8, 7, 5, 3, 2, 1], vec);
+    /// ```
+    pub fn insert_ord_by<F>(&mut self, item: A, mut f: F)
+    where
+        A: Ord,
+        F: FnMut(&A, &A) -> Ordering,
+    {
+        match self.binary_search_by(|scan_item| f(scan_item, &item)) {
+            Ok(idx) | Err(idx) => self.insert(idx, item),
+        }
+    }
+
+    /// Insert an element into a sorted vector where the comparison function
+    /// delegates to the Ord implementation for values calculated by a user-
+    /// provided function defined on the item type.
+    ///
+    /// This function assumes the vector is already sorted. If it isn't sorted,
+    /// this function may insert the provided value out of order.
+    ///
+    /// Note that the ordering of the sorted vector must logically match the
+    /// `PartialOrd` implementation of the type returned by the passed comparator
+    /// function `f`. Incompatible definitions of the ordering won't result in
+    /// memory unsafety, but will likely result in out-of-order insertions.
+    ///
+    ///
+    /// Time: O(log n)
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # #[macro_use] extern crate imbl;
+    /// use imbl::vector::Vector;
+    ///
+    /// type A = (u8, &'static str);
+    ///
+    /// let mut vec: Vector<A> = vector![(3, "a"), (1, "c"), (0, "d")];
+    ///
+    /// // For the sake of this example, let's say that only the second element
+    /// // of the A tuple is important in the context of comparison.
+    /// vec.insert_ord_by_key((0, "b"), |a| a.1);
+    /// assert_eq!(vector![(3, "a"), (0, "b"), (1, "c"), (0, "d")], vec);
+    ///
+    /// // Note that `insert_ord` does not work in this case because it uses
+    /// // the default comparison function for the item type.
+    /// vec.insert_ord((0, "e"));
+    /// assert_eq!(vector![(3, "a"), (0, "b"), (0, "e"), (1, "c"), (0, "d")], vec);
+    /// ```
+    pub fn insert_ord_by_key<B, F>(&mut self, item: A, mut f: F)
+    where
+        B: Ord,
+        F: FnMut(&A) -> B,
+    {
+        match self.binary_search_by_key(&f(&item), |scan_item| f(scan_item)) {
+            Ok(idx) | Err(idx) => self.insert(idx, item),
+        }
+    }
+
     /// Sort a vector.
     ///
     /// Time: O(n log n)
@@ -1499,25 +1621,11 @@ impl<A: Clone> Vector<A> {
             sort::quicksort(self.focus_mut(), &cmp);
         }
     }
-
-    /// Verify the internal consistency of a vector.
-    ///
-    /// This method walks the RRB tree making up the current `Vector`
-    /// (if it has one) and verifies that all the invariants hold.
-    /// If something is wrong, it will panic.
-    ///
-    /// This method requires the `debug` feature flag.
-    #[cfg(any(test, feature = "debug"))]
-    pub fn assert_invariants(&self) {
-        if let Full(_, ref tree) = self.vector {
-            tree.assert_invariants();
-        }
-    }
 }
 
 // Implementation details
 
-impl<A: Clone> RRB<A> {
+impl<A> RRB<A> {
     fn new(pool: &RRBPool<A>) -> Self {
         RRB {
             length: 0,
@@ -1538,7 +1646,9 @@ impl<A: Clone> RRB<A> {
             self.outer_f.len() + self.inner_f.len() + ml + self.inner_b.len() + self.outer_b.len()
         );
     }
+}
 
+impl<A: Clone> RRB<A> {
     fn prune(&mut self) {
         if self.middle.is_empty() {
             self.middle = Ref::new(Node::new());
@@ -1677,7 +1787,7 @@ fn replace_pool_def<A: PoolDefault>(pool: &Pool<A>, dest: &mut PoolRef<A>) -> Po
 
 // Core traits
 
-impl<A: Clone> Default for Vector<A> {
+impl<A> Default for Vector<A> {
     fn default() -> Self {
         Self::new()
     }
@@ -1698,7 +1808,7 @@ impl<A: Clone> Clone for Vector<A> {
     }
 }
 
-impl<A: Clone + Debug> Debug for Vector<A> {
+impl<A: Debug> Debug for Vector<A> {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
         f.debug_list().entries(self.iter()).finish()
         // match self {
@@ -1713,21 +1823,21 @@ impl<A: Clone + Debug> Debug for Vector<A> {
 }
 
 #[cfg(not(has_specialisation))]
-impl<A: Clone + PartialEq> PartialEq for Vector<A> {
+impl<A: PartialEq> PartialEq for Vector<A> {
     fn eq(&self, other: &Self) -> bool {
         self.len() == other.len() && self.iter().eq(other.iter())
     }
 }
 
 #[cfg(has_specialisation)]
-impl<A: Clone + PartialEq> PartialEq for Vector<A> {
+impl<A: PartialEq> PartialEq for Vector<A> {
     default fn eq(&self, other: &Self) -> bool {
         self.len() == other.len() && self.iter().eq(other.iter())
     }
 }
 
 #[cfg(has_specialisation)]
-impl<A: Clone + Eq> PartialEq for Vector<A> {
+impl<A: Eq> PartialEq for Vector<A> {
     fn eq(&self, other: &Self) -> bool {
         fn cmp_chunk<A>(left: &PoolRef<Chunk<A>>, right: &PoolRef<Chunk<A>>) -> bool {
             (left.is_empty() && right.is_empty()) || PoolRef::ptr_eq(left, right)
@@ -1765,21 +1875,21 @@ impl<A: Clone + Eq> PartialEq for Vector<A> {
     }
 }
 
-impl<A: Clone + Eq> Eq for Vector<A> {}
+impl<A: Eq> Eq for Vector<A> {}
 
-impl<A: Clone + PartialOrd> PartialOrd for Vector<A> {
+impl<A: PartialOrd> PartialOrd for Vector<A> {
     fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
         self.iter().partial_cmp(other.iter())
     }
 }
 
-impl<A: Clone + Ord> Ord for Vector<A> {
+impl<A: Ord> Ord for Vector<A> {
     fn cmp(&self, other: &Self) -> Ordering {
         self.iter().cmp(other.iter())
     }
 }
 
-impl<A: Clone + Hash> Hash for Vector<A> {
+impl<A: Hash> Hash for Vector<A> {
     fn hash<H: Hasher>(&self, state: &mut H) {
         for i in self {
             i.hash(state)
@@ -1835,7 +1945,7 @@ impl<A: Clone> Extend<A> for Vector<A> {
     }
 }
 
-impl<A: Clone> Index<usize> for Vector<A> {
+impl<A> Index<usize> for Vector<A> {
     type Output = A;
     /// Get a reference to the value at index `index` in the vector.
     ///
@@ -1867,7 +1977,7 @@ impl<A: Clone> IndexMut<usize> for Vector<A> {
 
 // Conversions
 
-impl<'a, A: Clone> IntoIterator for &'a Vector<A> {
+impl<'a, A> IntoIterator for &'a Vector<A> {
     type Item = &'a A;
     type IntoIter = Iter<'a, A>;
     fn into_iter(self) -> Self::IntoIter {
@@ -1909,6 +2019,15 @@ where
     }
 }
 
+impl<A, const N: usize> From<[A; N]> for Vector<A>
+where
+    A: Clone,
+{
+    fn from(arr: [A; N]) -> Self {
+        IntoIterator::into_iter(arr).collect()
+    }
+}
+
 impl<'a, A: Clone> From<&'a [A]> for Vector<A> {
     fn from(slice: &[A]) -> Self {
         slice.iter().cloned().collect()
@@ -1944,13 +2063,16 @@ impl<'a, A: Clone> From<&'a Vec<A>> for Vector<A> {
 /// To obtain one, use [`Vector::iter()`][iter].
 ///
 /// [iter]: enum.Vector.html#method.iter
+// TODO: we'd like to support Clone even if A is not Clone, but it isn't trivial because
+// the TreeFocus variant of Focus does need A to be Clone.
+#[derive(Clone)]
 pub struct Iter<'a, A> {
     focus: Focus<'a, A>,
     front_index: usize,
     back_index: usize,
 }
 
-impl<'a, A: Clone> Iter<'a, A> {
+impl<'a, A> Iter<'a, A> {
     fn new(seq: &'a Vector<A>) -> Self {
         Iter {
             focus: seq.focus(),
@@ -1968,7 +2090,7 @@ impl<'a, A: Clone> Iter<'a, A> {
     }
 }
 
-impl<'a, A: Clone> Iterator for Iter<'a, A> {
+impl<'a, A> Iterator for Iter<'a, A> {
     type Item = &'a A;
 
     /// Advance the iterator and return the next value.
@@ -1978,7 +2100,6 @@ impl<'a, A: Clone> Iterator for Iter<'a, A> {
         if self.front_index >= self.back_index {
             return None;
         }
-        #[allow(unsafe_code)]
         let focus: &'a mut Focus<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         let value = focus.get(self.front_index);
         self.front_index += 1;
@@ -1991,7 +2112,7 @@ impl<'a, A: Clone> Iterator for Iter<'a, A> {
     }
 }
 
-impl<'a, A: Clone> DoubleEndedIterator for Iter<'a, A> {
+impl<'a, A> DoubleEndedIterator for Iter<'a, A> {
     /// Advance the iterator and return the next value.
     ///
     /// Time: O(1)*
@@ -2000,15 +2121,14 @@ impl<'a, A: Clone> DoubleEndedIterator for Iter<'a, A> {
             return None;
         }
         self.back_index -= 1;
-        #[allow(unsafe_code)]
         let focus: &'a mut Focus<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         focus.get(self.back_index)
     }
 }
 
-impl<'a, A: Clone> ExactSizeIterator for Iter<'a, A> {}
+impl<'a, A> ExactSizeIterator for Iter<'a, A> {}
 
-impl<'a, A: Clone> FusedIterator for Iter<'a, A> {}
+impl<'a, A> FusedIterator for Iter<'a, A> {}
 
 /// A mutable iterator over vectors with values of type `A`.
 ///
@@ -2021,10 +2141,17 @@ pub struct IterMut<'a, A> {
     back_index: usize,
 }
 
-impl<'a, A> IterMut<'a, A>
-where
-    A: Clone,
-{
+impl<'a, A> IterMut<'a, A> {
+    fn from_focus(focus: FocusMut<'a, A>) -> Self {
+        IterMut {
+            front_index: 0,
+            back_index: focus.len(),
+            focus,
+        }
+    }
+}
+
+impl<'a, A: Clone> IterMut<'a, A> {
     fn new(seq: &'a mut Vector<A>) -> Self {
         let focus = seq.focus_mut();
         let len = focus.len();
@@ -2034,14 +2161,6 @@ where
             back_index: len,
         }
     }
-
-    fn from_focus(focus: FocusMut<'a, A>) -> Self {
-        IterMut {
-            front_index: 0,
-            back_index: focus.len(),
-            focus,
-        }
-    }
 }
 
 impl<'a, A> Iterator for IterMut<'a, A>
@@ -2057,7 +2176,6 @@ where
         if self.front_index >= self.back_index {
             return None;
         }
-        #[allow(unsafe_code)]
         let focus: &'a mut FocusMut<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         let value = focus.get_mut(self.front_index);
         self.front_index += 1;
@@ -2082,7 +2200,6 @@ where
             return None;
         }
         self.back_index -= 1;
-        #[allow(unsafe_code)]
         let focus: &'a mut FocusMut<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         focus.get_mut(self.back_index)
     }
@@ -2097,7 +2214,7 @@ pub struct ConsumingIter<A> {
     vector: Vector<A>,
 }
 
-impl<A: Clone> ConsumingIter<A> {
+impl<A> ConsumingIter<A> {
     fn new(vector: Vector<A>) -> Self {
         Self { vector }
     }
@@ -2143,7 +2260,7 @@ pub struct Chunks<'a, A> {
     back_index: usize,
 }
 
-impl<'a, A: Clone> Chunks<'a, A> {
+impl<'a, A> Chunks<'a, A> {
     fn new(seq: &'a Vector<A>) -> Self {
         Chunks {
             focus: seq.focus(),
@@ -2153,7 +2270,7 @@ impl<'a, A: Clone> Chunks<'a, A> {
     }
 }
 
-impl<'a, A: Clone> Iterator for Chunks<'a, A> {
+impl<'a, A> Iterator for Chunks<'a, A> {
     type Item = &'a [A];
 
     /// Advance the iterator and return the next value.
@@ -2163,7 +2280,6 @@ impl<'a, A: Clone> Iterator for Chunks<'a, A> {
         if self.front_index >= self.back_index {
             return None;
         }
-        #[allow(unsafe_code)]
         let focus: &'a mut Focus<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         let (range, value) = focus.chunk_at(self.front_index);
         self.front_index = range.end;
@@ -2171,7 +2287,7 @@ impl<'a, A: Clone> Iterator for Chunks<'a, A> {
     }
 }
 
-impl<'a, A: Clone> DoubleEndedIterator for Chunks<'a, A> {
+impl<'a, A> DoubleEndedIterator for Chunks<'a, A> {
     /// Remove and return an element from the back of the iterator.
     ///
     /// Time: O(1)*
@@ -2180,7 +2296,6 @@ impl<'a, A: Clone> DoubleEndedIterator for Chunks<'a, A> {
             return None;
         }
         self.back_index -= 1;
-        #[allow(unsafe_code)]
         let focus: &'a mut Focus<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         let (range, value) = focus.chunk_at(self.back_index);
         self.back_index = range.start;
@@ -2188,7 +2303,7 @@ impl<'a, A: Clone> DoubleEndedIterator for Chunks<'a, A> {
     }
 }
 
-impl<'a, A: Clone> FusedIterator for Chunks<'a, A> {}
+impl<'a, A> FusedIterator for Chunks<'a, A> {}
 
 /// A mutable iterator over the leaf nodes of a vector.
 ///
@@ -2222,7 +2337,6 @@ impl<'a, A: Clone> Iterator for ChunksMut<'a, A> {
         if self.front_index >= self.back_index {
             return None;
         }
-        #[allow(unsafe_code)]
         let focus: &'a mut FocusMut<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         let (range, value) = focus.chunk_at(self.front_index);
         self.front_index = range.end;
@@ -2239,7 +2353,6 @@ impl<'a, A: Clone> DoubleEndedIterator for ChunksMut<'a, A> {
             return None;
         }
         self.back_index -= 1;
-        #[allow(unsafe_code)]
         let focus: &'a mut FocusMut<'a, A> = unsafe { &mut *(&mut self.focus as *mut _) };
         let (range, value) = focus.chunk_at(self.back_index);
         self.back_index = range.start;
@@ -2269,6 +2382,11 @@ mod test {
     use ::proptest::collection::vec;
     use ::proptest::num::{i32, usize};
     use ::proptest::proptest;
+    use static_assertions::{assert_impl_all, assert_not_impl_any};
+
+    assert_impl_all!(Vector<i32>: Send, Sync);
+    assert_not_impl_any!(Vector<*const i32>: Send, Sync);
+    assert_covariant!(Vector<T> in T);
 
     #[test]
     fn macro_allows_trailing_comma() {
@@ -2335,7 +2453,7 @@ mod test {
         assert_eq!(Some(&4098), l.get(1));
         assert_eq!(Some(&4097), l.get(2));
         let len = l.len();
-        l.slice(2..len);
+        let _ = l.slice(2..len);
     }
 
     #[test]
@@ -2347,6 +2465,10 @@ mod test {
 
     #[test]
     fn issue_70() {
+        // This test assumes that chunks are of size 64.
+        if CHUNK_SIZE != 64 {
+            return;
+        }
         let mut x = Vector::new();
         for _ in 0..262 {
             x.push_back(0);
@@ -2426,7 +2548,9 @@ mod test {
         x.push_back(0u32);
         match x.vector {
             VectorInner::Full(_, tree) => {
-                assert_eq!(3, tree.middle.number_of_children());
+                if CHUNK_SIZE == 64 {
+                    assert_eq!(3, tree.middle.number_of_children());
+                }
                 assert_eq!(
                     2 * NODE_SIZE * CHUNK_SIZE + CHUNK_SIZE - 1,
                     tree.middle.len()
